@@ -1,6 +1,6 @@
 import html
 import json
-import string
+import typing
 import warnings
 from urllib.parse import urlparse
 
@@ -33,9 +33,11 @@ from django.utils.translation import gettext_lazy as _
 
 from allauth import app_settings as allauth_app_settings
 from allauth.account import app_settings, signals
-from allauth.core import context, ratelimit
+from allauth.core import context
+from allauth.core.internal import ratelimit
 from allauth.core.internal.adapter import BaseAdapter
-from allauth.core.internal.httpkit import headed_redirect_response
+from allauth.core.internal.cryptokit import generate_user_code
+from allauth.core.internal.httpkit import headed_redirect_response, is_headless_request
 from allauth.utils import generate_unique_username, import_attribute
 
 
@@ -58,17 +60,23 @@ class DefaultAccountAdapter(BaseAdapter):
         "email_password_mismatch": _(
             "The email address and/or password you specified are not correct."
         ),
+        "phone_password_mismatch": _(
+            "The phone number and/or password you specified are not correct."
+        ),
         "email_taken": _("A user is already registered with this email address."),
         "enter_current_password": _("Please type your current password."),
         "incorrect_code": _("Incorrect code."),
         "incorrect_password": _("Incorrect password."),
         "invalid_or_expired_key": _("Invalid or expired key."),
+        "invalid_login": _("Invalid login."),
         "invalid_password_reset": _("The password reset token was invalid."),
         "max_email_addresses": _("You cannot add more than %d email addresses."),
+        "phone_taken": _("A user is already registered with this phone number."),
         "too_many_login_attempts": _(
             "Too many failed login attempts. Try again later."
         ),
-        "unknown_email": _("The email address is not assigned to any user account"),
+        "unknown_email": _("The email address is not assigned to any user account."),
+        "unknown_phone": _("The phone number is not assigned to any user account."),
         "unverified_primary_email": _("Your primary email address must be verified."),
         "username_blacklisted": _(
             "Username can not be used. Please use other username."
@@ -79,6 +87,9 @@ class DefaultAccountAdapter(BaseAdapter):
         "username_taken": AbstractUser._meta.get_field("username").error_messages[
             "unique"
         ],
+        "select_only_one": _("Please select only one."),
+        "same_as_current": _("The new value must be different from the current one."),
+        "rate_limited": _("Be patient, you are sending too many requests."),
     }
 
     def stash_verified_email(self, request, email):
@@ -101,7 +112,10 @@ class DefaultAccountAdapter(BaseAdapter):
             ret = verified_email.lower() == email.lower()
         return ret
 
-    def can_delete_email(self, email_address):
+    def can_delete_email(self, email_address) -> bool:
+        """
+        Returns whether or not the given email address can be deleted.
+        """
         from allauth.account.models import EmailAddress
 
         has_other = (
@@ -109,10 +123,7 @@ class DefaultAccountAdapter(BaseAdapter):
             .exclude(pk=email_address.pk)
             .exists()
         )
-        login_by_email = (
-            app_settings.AUTHENTICATION_METHOD
-            == app_settings.AuthenticationMethod.EMAIL
-        )
+        login_by_email = app_settings.LOGIN_METHODS == {app_settings.LoginMethod.EMAIL}
         if email_address.primary:
             if has_other:
                 # Don't allow, let the user mark one of the others as primary
@@ -131,7 +142,10 @@ class DefaultAccountAdapter(BaseAdapter):
         else:
             return True
 
-    def format_email_subject(self, subject):
+    def format_email_subject(self, subject) -> str:
+        """
+        Formats the given email subject.
+        """
         prefix = app_settings.EMAIL_SUBJECT_PREFIX
         if prefix is None:
             site = get_current_site(context.request)
@@ -185,16 +199,21 @@ class DefaultAccountAdapter(BaseAdapter):
             msg.content_subtype = "html"  # Main content is now text/html
         return msg
 
-    def send_mail(self, template_prefix, email, context):
+    def send_mail(self, template_prefix: str, email: str, context: dict) -> None:
+        request = globals()["context"].request
         ctx = {
+            "request": request,
             "email": email,
-            "current_site": get_current_site(globals()["context"].request),
+            "current_site": get_current_site(request),
         }
         ctx.update(context)
         msg = self.render_mail(template_prefix, email, ctx)
         msg.send()
 
     def get_signup_redirect_url(self, request):
+        """
+        Returns the default URL to redirect to directly after signing up.
+        """
         return resolve_url(app_settings.SIGNUP_REDIRECT_URL)
 
     def get_login_redirect_url(self, request):
@@ -203,7 +222,7 @@ class DefaultAccountAdapter(BaseAdapter):
         that URLs passed explicitly (e.g. by passing along a `next`
         GET parameter) take precedence over the value returned here.
         """
-        assert request.user.is_authenticated
+        assert request.user.is_authenticated  # nosec
         url = getattr(settings, "LOGIN_REDIRECT_URLNAME", None)
         if url:
             warnings.warn(
@@ -315,9 +334,11 @@ class DefaultAccountAdapter(BaseAdapter):
             user.set_unusable_password()
         self.populate_username(request, user)
         if commit:
-            # Ability not to commit makes it easier to derive from
-            # this adapter by adding
             user.save()
+        if getattr(form, "_has_phone_field", False):
+            phone = form.cleaned_data.get("phone")
+            if phone:
+                self.set_phone(user, phone, False)
         return user
 
     def clean_username(self, username, shallow=False):
@@ -343,7 +364,7 @@ class DefaultAccountAdapter(BaseAdapter):
                 raise self.validation_error("username_taken")
         return username
 
-    def clean_email(self, email):
+    def clean_email(self, email: str) -> str:
         """
         Validates an email value. You can hook into this if you want to
         (dynamically) restrict what email addresses can be chosen.
@@ -361,6 +382,13 @@ class DefaultAccountAdapter(BaseAdapter):
         validate_password(password, user)
         return password
 
+    def clean_phone(self, phone: str) -> str:
+        """
+        Validates a phone number. You can hook into this if you want to
+        (dynamically) restrict what phone numbers can be chosen.
+        """
+        return phone
+
     def validate_unique_email(self, email):
         return email
 
@@ -377,7 +405,7 @@ class DefaultAccountAdapter(BaseAdapter):
         Wrapper of `django.contrib.messages.add_message`, that reads
         the message text from a template.
         """
-        if getattr(getattr(request, "allauth", None), "headless", None):
+        if is_headless_request(request):
             return
         if "django.contrib.messages" in settings.INSTALLED_APPS:
             if message:
@@ -453,7 +481,7 @@ class DefaultAccountAdapter(BaseAdapter):
         signal_kwargs,
         email,
         signup,
-        redirect_url
+        redirect_url,
     ):
         if not user.is_active:
             return self.respond_user_inactive(request, user)
@@ -467,13 +495,18 @@ class DefaultAccountAdapter(BaseAdapter):
         signal_kwargs,
         email,
         signup,
-        redirect_url
+        redirect_url,
     ):
         from .utils import get_login_redirect_url
 
-        response = HttpResponseRedirect(
-            get_login_redirect_url(request, redirect_url, signup=signup)
-        )
+        if is_headless_request(request):
+            from allauth.headless.base.response import AuthenticationResponse
+
+            response = AuthenticationResponse(request)
+        else:
+            response = HttpResponseRedirect(
+                get_login_redirect_url(request, redirect_url, signup=signup)
+            )
 
         if signal_kwargs is None:
             signal_kwargs = {}
@@ -523,7 +556,10 @@ class DefaultAccountAdapter(BaseAdapter):
 
         return email_verification.verify_email(request, email_address)
 
-    def set_password(self, user, password):
+    def set_password(self, user, password) -> None:
+        """
+        Sets the password for the user.
+        """
         user.set_password(password)
         user.save()
 
@@ -550,6 +586,13 @@ class DefaultAccountAdapter(BaseAdapter):
         # get_host already validates the given host, so no need to check it again
         allowed_hosts = {context.request.get_host()} | set(settings.ALLOWED_HOSTS)
 
+        # Include hosts derived from CSRF_TRUSTED_ORIGINS
+        trusted_hosts = {
+            urlparse(origin).netloc for origin in settings.CSRF_TRUSTED_ORIGINS
+        }
+        allowed_hosts.update(trusted_hosts)
+
+        # Handle wildcard case
         if "*" in allowed_hosts:
             parsed_host = urlparse(url).netloc
             allowed_host = {parsed_host} if parsed_host else None
@@ -589,15 +632,10 @@ class DefaultAccountAdapter(BaseAdapter):
             request, emailconfirmation
         )
 
-    def should_send_confirmation_mail(self, request, email_address, signup):
-        send_email = ratelimit.consume(
-            request,
-            action="confirm_email",
-            key=email_address.email.lower(),
-        )
-        return send_email
+    def should_send_confirmation_mail(self, request, email_address, signup) -> bool:
+        return True
 
-    def send_account_already_exists_mail(self, email):
+    def send_account_already_exists_mail(self, email: str) -> None:
         from allauth.account.internal import flows
 
         signup_url = flows.signup.get_signup_url(context.request)
@@ -605,7 +643,6 @@ class DefaultAccountAdapter(BaseAdapter):
             context.request
         )
         ctx = {
-            "request": context.request,
             "signup_url": signup_url,
             "password_reset_url": password_reset_url,
         }
@@ -645,15 +682,30 @@ class DefaultAccountAdapter(BaseAdapter):
 
     def _delete_login_attempts_cached_email(self, request, **credentials):
         cache_key = self._get_login_attempts_cache_key(request, **credentials)
-        ratelimit.clear(request, action="login_failed", key=cache_key)
+        # Here, we wipe the login failed rate limit, completely. This is safe,
+        # as we only do this on a succesful password reset, which is rate limited
+        # on itself (e.g. sending of email etc.).
+        ratelimit.clear(
+            request,
+            config=app_settings.RATE_LIMITS,
+            action="login_failed",
+            key=cache_key,
+        )
+
+    def _rollback_login_failed_rl_usage(self) -> None:
+        usage = getattr(self, "_login_failed_rl_usage", None)
+        if usage:
+            usage.rollback()
 
     def pre_authenticate(self, request, **credentials):
         cache_key = self._get_login_attempts_cache_key(request, **credentials)
-        if not ratelimit.consume(
+        self._login_failed_rl_usage = ratelimit.consume(
             request,
+            config=app_settings.RATE_LIMITS,
             action="login_failed",
             key=cache_key,
-        ):
+        )
+        if not self._login_failed_rl_usage:
             raise self.validation_error("too_many_login_attempts")
 
     def authenticate(self, request, **credentials):
@@ -666,7 +718,12 @@ class DefaultAccountAdapter(BaseAdapter):
         alt_user = AuthenticationBackend.unstash_authenticated_user()
         user = user or alt_user
         if user:
-            self._delete_login_attempts_cached_email(request, **credentials)
+            # On a succesful login, we cannot just wipe the login failed rate
+            # limit. That consists of 2 parts, a per IP limit, and, a per
+            # key(email) limit. Wiping it completely would allow an attacker to
+            # insert periodic successful logins during a brute force
+            # process. So instead, we are rolling back our consumption.
+            self._rollback_login_failed_rl_usage()
         else:
             self.authentication_failed(request, **credentials)
         return user
@@ -685,6 +742,10 @@ class DefaultAccountAdapter(BaseAdapter):
         email = EmailAddress.objects.get_primary_email(user)
         if email:
             credentials["email"] = email
+        if app_settings.LoginMethod.PHONE in app_settings.LOGIN_METHODS:
+            phone_verified = self.get_phone(user)
+            if phone_verified:
+                credentials["phone"] = phone_verified[0]
         reauth_user = self.authenticate(context.request, **credentials)
         return reauth_user is not None and reauth_user.pk == user.pk
 
@@ -697,7 +758,7 @@ class DefaultAccountAdapter(BaseAdapter):
             ]
         )
 
-    def get_client_ip(self, request):
+    def get_client_ip(self, request) -> str:
         x_forwarded_for = request.META.get("HTTP_X_FORWARDED_FOR")
         if x_forwarded_for:
             ip = x_forwarded_for.split(",")[0]
@@ -715,9 +776,17 @@ class DefaultAccountAdapter(BaseAdapter):
     def get_login_stages(self):
         ret = []
         ret.append("allauth.account.stages.LoginByCodeStage")
+        ret.append("allauth.account.stages.PhoneVerificationStage")
         ret.append("allauth.account.stages.EmailVerificationStage")
         if allauth_app_settings.MFA_ENABLED:
+            from allauth.mfa import app_settings as mfa_settings
+
             ret.append("allauth.mfa.stages.AuthenticateStage")
+            if mfa_settings.TRUST_ENABLED:
+                ret.append("allauth.mfa.stages.TrustStage")
+
+            if mfa_settings.PASSKEY_SIGNUP_ENABLED:
+                ret.append("allauth.mfa.webauthn.stages.PasskeySignupStage")
         return ret
 
     def get_reauthentication_methods(self, user):
@@ -774,17 +843,28 @@ class DefaultAccountAdapter(BaseAdapter):
         self.send_mail(template_prefix, email, ctx)
 
     def generate_login_code(self) -> str:
-        return self._generate_code()
+        """
+        Generates a new login code.
+        """
+        return generate_user_code()
+
+    def generate_password_reset_code(self) -> str:
+        """
+        Generates a new password reset code.
+        """
+        return generate_user_code(length=8)
 
     def generate_email_verification_code(self) -> str:
-        return self._generate_code()
+        """
+        Generates a new email verification code.
+        """
+        return generate_user_code()
 
-    def _generate_code(self):
-        forbidden_chars = "0OI18B2ZAEU"
-        allowed_chars = string.ascii_uppercase + string.digits
-        for ch in forbidden_chars:
-            allowed_chars = allowed_chars.replace(ch, "")
-        return get_random_string(length=6, allowed_chars=allowed_chars)
+    def generate_phone_verification_code(self) -> str:
+        """
+        Generates a new phone verification code.
+        """
+        return generate_user_code()
 
     def is_login_by_code_required(self, login) -> bool:
         """
@@ -806,6 +886,77 @@ class DefaultAccountAdapter(BaseAdapter):
             return False
         return method is None or method in value
 
+    def phone_form_field(self, **kwargs):
+        """
+        Returns a form field used to input phone numbers.
+        """
+        from allauth.account.fields import PhoneField
 
-def get_adapter(request=None):
+        return PhoneField(**kwargs)
+
+    def send_unknown_account_sms(self, phone: str, **kwargs) -> None:
+        """
+        In case enumeration prevention is enabled, and, a verification code
+        is requested for an unlisted phone number, this method is invoked to
+        send a text explaining that no account is on file.
+        """
+        pass
+
+    def send_account_already_exists_sms(self, phone: str) -> None:
+        pass
+
+    def send_verification_code_sms(self, user, phone: str, code: str, **kwargs):
+        """
+        Sends a verification code.
+        """
+        raise NotImplementedError
+
+    @property
+    def _has_phone_impl(self) -> bool:
+        """
+        Checks whether the phone number adapter is fully implemented.
+        """
+        methods = (
+            "send_verification_code_sms",
+            "set_phone",
+            "get_phone",
+            "set_phone_verified",
+            "get_user_by_phone",
+        )
+        return all(
+            getattr(self.__class__, method) != getattr(DefaultAccountAdapter, method)
+            for method in methods
+        )
+
+    def set_phone(self, user, phone: str, verified: bool):
+        """
+        Sets the phone number (and verified status) for the given user.
+        """
+        raise NotImplementedError
+
+    def get_phone(self, user) -> typing.Optional[typing.Tuple[str, bool]]:
+        """
+        Returns the phone number stored for the given user. A tuple of the
+        phone number itself, and whether or not the phone number was verified is
+        returned.
+        """
+        raise NotImplementedError
+
+    def set_phone_verified(self, user, phone: str):
+        """
+        Marks the specified phone number for the given user as
+        verified. Note that the user is already expected to have
+        the phone number attached to the account.
+        """
+        raise NotImplementedError
+
+    def get_user_by_phone(self, phone: str):
+        """
+        Looks up a user given the specified phone number. Returns ``None`` if no user
+        was found.
+        """
+        raise NotImplementedError
+
+
+def get_adapter(request=None) -> DefaultAccountAdapter:
     return import_attribute(app_settings.ADAPTER)(request)
