@@ -8,7 +8,7 @@ from allauth.socialaccount.providers.oauth2.views import (
 
 class ExistOAuth2Adapter(OAuth2Adapter):
     provider_id = "exist"
-    access_token_url = "https://exist.io/oauth2/access_token"
+    access_token_url = "https://exist.io/oauth2/access_token"  # nosec
     authorize_url = "https://exist.io/oauth2/authorize"
     profile_url = "https://exist.io/api/2/accounts/profile/"
 
