@@ -1,4 +1,5 @@
 from allauth.socialaccount.providers.base import ProviderAccount
+from allauth.socialaccount.providers.foursquare.views import FoursquareOAuth2Adapter
 from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider
 
 
@@ -10,14 +11,15 @@ class FoursquareAccount(ProviderAccount):
         return self.account.extra_data.get("photo")
 
     def to_str(self):
-        dflt = super(FoursquareAccount, self).to_str()
-        return self.account.extra_data.get("name", dflt)
+        dflt = super().to_str()
+        return self.account.extra_data.get("contact", {}).get("email", dflt)
 
 
 class FoursquareProvider(OAuth2Provider):
     id = "foursquare"
     name = "Foursquare"
     account_class = FoursquareAccount
+    oauth2_adapter_class = FoursquareOAuth2Adapter
 
     def extract_uid(self, data):
         return str(data["id"])
