@@ -1,7 +1,4 @@
-# -*- coding: utf-8 -*-
-from allauth.socialaccount.providers.lemonldap.provider import (
-    LemonLDAPProvider,
-)
+from allauth.socialaccount.providers.lemonldap.provider import LemonLDAPProvider
 from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns
 
 
