@@ -1,16 +1,18 @@
 from allauth.socialaccount.providers.base import AuthAction, ProviderAccount
 from allauth.socialaccount.providers.oauth.provider import OAuthProvider
+from allauth.socialaccount.providers.twitter.views import TwitterOAuthAdapter
 
 
 class TwitterAccount(ProviderAccount):
     def get_screen_name(self):
+        """The screen name is the username of the Twitter account."""
         return self.account.extra_data.get("screen_name")
 
     def get_profile_url(self):
         ret = None
         screen_name = self.get_screen_name()
         if screen_name:
-            ret = "http://twitter.com/" + screen_name
+            ret = "http://x.com/" + screen_name
         return ret
 
     def get_avatar_url(self):
@@ -29,18 +31,19 @@ class TwitterAccount(ProviderAccount):
 
 class TwitterProvider(OAuthProvider):
     id = "twitter"
-    name = "Twitter"
+    name = "X"
     account_class = TwitterAccount
+    oauth_adapter_class = TwitterOAuthAdapter
 
     def get_auth_url(self, request, action):
         if action == AuthAction.REAUTHENTICATE:
-            url = "https://api.twitter.com/oauth/authorize"
+            url = "https://api.x.com/oauth/authorize"
         else:
-            url = "https://api.twitter.com/oauth/authenticate"
+            url = "https://api.x.com/oauth/authenticate"
         return url
 
     def extract_uid(self, data):
-        return data["id"]
+        return str(data["id"])
 
     def extract_common_fields(self, data):
         return dict(
