@@ -1,4 +1,5 @@
 from django import template
+from django.contrib.auth import REDIRECT_FIELD_NAME
 from django.utils.safestring import mark_safe
 
 from allauth.socialaccount.adapter import get_adapter
@@ -26,15 +27,15 @@ def provider_login_url(context, provider, **params):
         del query["scope"]
     if auth_params == "":
         del query["auth_params"]
-    if "next" not in query:
-        next = get_request_param(request, "next")
+    if REDIRECT_FIELD_NAME not in query:
+        next = get_request_param(request, REDIRECT_FIELD_NAME)
         if next:
-            query["next"] = next
+            query[REDIRECT_FIELD_NAME] = next
         elif process == "redirect":
-            query["next"] = request.get_full_path()
+            query[REDIRECT_FIELD_NAME] = request.get_full_path()
     else:
-        if not query["next"]:
-            del query["next"]
+        if not query[REDIRECT_FIELD_NAME]:
+            del query[REDIRECT_FIELD_NAME]
     # get the login url and append query as url parameters
     return provider.get_login_url(request, **query)
 
@@ -44,7 +45,7 @@ def providers_media_js(context):
     request = context["request"]
     providers = get_adapter().list_providers(request)
     ret = "\n".join(p.media_js(request) for p in providers)
-    return mark_safe(ret)
+    return mark_safe(ret)  # nosec
 
 
 @register.simple_tag
@@ -77,4 +78,9 @@ def get_providers(context):
     request = context["request"]
     adapter = get_adapter()
     providers = adapter.list_providers(request)
+    providers = [
+        provider
+        for provider in providers
+        if (not provider.uses_apps or not provider.app.settings.get("hidden"))
+    ]
     return sorted(providers, key=lambda p: p.name)
